@@ -1,7 +1,6 @@
 /*
  * pam_tally.c
  *
- * $Id: pam_tally.c,v 1.24 2006/05/02 11:12:00 kukuk Exp $
  */
 
 
@@ -11,15 +10,12 @@
  * Stuff stolen from pam_rootok and pam_listfile
  *
  * Changes by Tomas Mraz <tmraz@redhat.com> 5 January 2005
- * Audit option added for Tomas patch by Sebastien Tricaud <toady@gscore.org> 13 January 2005
+ * Audit option added for Tomas patch by
+ * Sebastien Tricaud <toady@gscore.org> 13 January 2005
  */
 
 #include "config.h"
 
-#if defined(MAIN) && defined(MEMORY_DEBUG)
-# undef exit
-#endif /* defined(MAIN) && defined(MEMORY_DEBUG) */
-
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
@@ -34,11 +30,6 @@
 #include <sys/param.h>
 #include "faillog.h"
 
-#ifndef TRUE
-#define TRUE  1L
-#define FALSE 0L
-#endif
-
 /*
  * here, we make a definition for the externally accessible function
  * in this file (this definition is required for static a module
@@ -46,14 +37,25 @@
  * modules include file to define the function prototypes.
  */
 
+#ifndef MAIN
 #define PAM_SM_AUTH
 #define PAM_SM_ACCOUNT
 /* #define PAM_SM_SESSION  */
 /* #define PAM_SM_PASSWORD */
 
-#include <security/pam_modules.h>
 #include <security/pam_modutil.h>
 #include <security/pam_ext.h>
+#endif
+#include <security/pam_modules.h>
+
+#ifndef TRUE
+#define TRUE  1L
+#define FALSE 0L
+#endif
+
+#ifndef HAVE_FSEEKO
+#define fseeko fseek
+#endif
 
 /*---------------------------------------------------------------------*/
 
@@ -64,8 +66,6 @@
 #define TALLY_FMT  "%hu"
 #define TALLY_HI   ((tally_t)~0L)
 
-#define UID_FMT    "%hu"
-
 #ifndef FILENAME_MAX
 # define FILENAME_MAX MAXPATHLEN
 #endif
@@ -96,7 +96,7 @@ struct tally_options {
 #define OPT_PER_USER			 010
 #define	OPT_NO_LOCK_TIME		 020
 #define OPT_NO_RESET			 040
-#define OPT_AUDIT                        100
+#define OPT_AUDIT                       0100
 
 
 /*---------------------------------------------------------------------*/
@@ -117,6 +117,9 @@ tally_log (const pam_handle_t *pamh UNUSED, int priority UNUSED,
 	fprintf(stderr,"\n");
 	va_end(args);
 }
+
+#define pam_modutil_getpwnam(pamh,user) getpwnam(user)
+
 #endif
 
 /*---------------------------------------------------------------------*/
@@ -230,7 +233,10 @@ pam_get_uid(pam_handle_t *pamh, uid_t *uid, const char **userp, struct tally_opt
 #ifdef MAIN
     user = cline_user;
 #else
-    pam_get_user( pamh, &user, NULL );
+    if ((pam_get_user( pamh, &user, NULL )) != PAM_SUCCESS) {
+      pam_syslog(pamh, LOG_ERR, "pam_get_user; user?");
+      return PAM_AUTH_ERR;
+    }
 #endif
 
     if ( !user || !*user ) {
@@ -287,7 +293,8 @@ tally_get_data( pam_handle_t *pamh, time_t *oldtime )
     }
     else {
       rv = -1;
-      *oldtime = 0;
+      if (oldtime)
+	*oldtime = 0;
     }
     return rv;
 }
@@ -335,7 +342,7 @@ get_tally(pam_handle_t *pamh, tally_t *tally, uid_t uid,
     }
 
     if ( ! ( *TALLY = fopen(filename,(*tally!=TALLY_HI)?"r+":"r") ) ) {
-      pam_syslog(pamh, LOG_ALERT, "Error opening %s for update", filename);
+      pam_syslog(pamh, LOG_ALERT, "Error opening %s for %s", filename, *tally!=TALLY_HI?"update":"read");
 
 /* Discovering why account service fails: e/uid are target user.
  *
@@ -345,13 +352,13 @@ get_tally(pam_handle_t *pamh, tally_t *tally, uid_t uid,
       return PAM_AUTH_ERR;
     }
 
-    if ( fseek( *TALLY, uid * sizeof(struct faillog), SEEK_SET ) ) {
+    if ( fseeko( *TALLY, (off_t) uid * sizeof(struct faillog), SEEK_SET ) ) {
           pam_syslog(pamh, LOG_ALERT, "fseek failed for %s", filename);
           fclose(*TALLY);
           return PAM_AUTH_ERR;
     }
 
-    if ( fileinfo.st_size <= uid * sizeof(struct faillog) ) {
+    if ( (size_t)fileinfo.st_size <= uid * sizeof(struct faillog) ) {
 
 	memset(fsp, 0, sizeof(struct faillog));
 	*tally=0;
@@ -382,7 +389,7 @@ set_tally(pam_handle_t *pamh, tally_t tally, uid_t uid,
     int retval = PAM_SUCCESS;
 
     if ( tally!=TALLY_HI ) {
-      if ( fseek( *TALLY, uid * sizeof(struct faillog), SEEK_SET ) ) {
+      if ( fseeko( *TALLY, (off_t) uid * sizeof(struct faillog), SEEK_SET ) ) {
 	pam_syslog(pamh, LOG_ALERT, "fseek failed for %s", filename);
 	retval = PAM_AUTH_ERR;
       } else {
@@ -489,7 +496,7 @@ tally_check (time_t oldtime, pam_handle_t *pamh, uid_t uid,
   tally_t
     deny          = opts->deny;
   tally_t
-    tally         = 0;  /* !TALLY_HI --> Log opened for update */
+    tally         = TALLY_HI;
   long
     lock_time     = opts->lock_time;
 
@@ -500,6 +507,10 @@ tally_check (time_t oldtime, pam_handle_t *pamh, uid_t uid,
     i=get_tally(pamh, &tally, uid, opts->filename, &TALLY, fsp);
     if ( i != PAM_SUCCESS ) { RETURN_ERROR( i ); }
 
+    if ( TALLY != NULL ) {
+      fclose(TALLY);
+    }
+
     if ( !(opts->ctrl & OPT_MAGIC_ROOT) || getuid() ) {       /* magic_root skips tally check */
 
       /* To deny or not to deny; that is the question */
@@ -518,9 +529,9 @@ tally_check (time_t oldtime, pam_handle_t *pamh, uid_t uid,
       	if ( lock_time + oldtime > time(NULL) )
       	{
       		pam_syslog(pamh, LOG_NOTICE,
-			 "user %s ("UID_FMT") has time limit [%lds left]"
+			 "user %s (%lu) has time limit [%lds left]"
 			 " since last failure.",
-			 user,uid,
+			 user, (unsigned long int) uid,
 			 oldtime+lock_time
 			 -time(NULL));
       		return PAM_AUTH_ERR;
@@ -539,8 +550,8 @@ tally_check (time_t oldtime, pam_handle_t *pamh, uid_t uid,
         ( ((opts->ctrl & OPT_DENY_ROOT) || uid) )    /* even_deny stops uid check    */
         ) {
         pam_syslog(pamh, LOG_NOTICE,
-		   "user %s ("UID_FMT") tally "TALLY_FMT", deny "TALLY_FMT,
-		   user, uid, tally, deny);
+		   "user %s (%lu) tally "TALLY_FMT", deny "TALLY_FMT,
+		   user, (unsigned long int) uid, tally, deny);
         return PAM_AUTH_ERR;                 /* Only unconditional failure   */
       }
     }
@@ -807,7 +818,8 @@ int main ( int argc UNUSED, char **argv )
     }
 
     if ( !cline_quiet )
-      printf("User %s\t("UID_FMT")\t%s "TALLY_FMT"\n",cline_user,uid,
+      printf("User %s\t(%lu)\t%s "TALLY_FMT"\n",cline_user,
+	     (unsigned long int) uid,
              (cline_reset!=TALLY_HI)?"had":"has",tally);
 
     i=set_tally(NULL, cline_reset, uid, cline_filename, &TALLY, fsp);
@@ -832,11 +844,13 @@ int main ( int argc UNUSED, char **argv )
       tally = fsp->fs_faillog.fail_cnt;
 
       if ( ( pw=getpwuid(uid) ) ) {
-        printf("User %s\t("UID_FMT")\t%s "TALLY_FMT"\n",pw->pw_name,uid,
+        printf("User %s\t(%lu)\t%s "TALLY_FMT"\n",pw->pw_name,
+	       (unsigned long int) uid,
                (cline_reset!=TALLY_HI)?"had":"has",tally);
       }
       else {
-        printf("User [NONAME]\t("UID_FMT")\t%s "TALLY_FMT"\n",uid,
+        printf("User [NONAME]\t(%lu)\t%s "TALLY_FMT"\n",
+	       (unsigned long int) uid,
                (cline_reset!=TALLY_HI)?"had":"has",tally);
       }
     }
