@@ -9,7 +9,6 @@
 #include <string.h>
 #include <sys/types.h>
 #include <unistd.h>
-#include <limits.h>
 #include <pwd.h>
 #include <shadow.h>
 #include <syslog.h>
@@ -20,16 +19,13 @@
 #include <sys/time.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#ifdef HAVE_LIBXCRYPT
-#include <xcrypt.h>
-#elif defined(HAVE_CRYPT_H)
+#ifdef HAVE_CRYPT_H
 #include <crypt.h>
 #endif
 
 #include "md5.h"
 #include "bigcrypt.h"
 #include "passverify.h"
-#include "cluster.h"
 
 #ifdef WITH_SELINUX
 #include <selinux/selinux.h>
@@ -39,18 +35,17 @@
 #endif
 
 #ifdef HELPER_COMPILE
+#define pam_modutil_getpwnam(h,n) getpwnam(n)
+#define pam_modutil_getspnam(h,n) getspnam(n)
 #define pam_syslog(h,a,b,c) helper_log_err(a,b,c)
-#define fgetpwnam(h,f,n) _fgetpwnam(f,n)
-#define fgetspnam(h,f,n) _fgetspnam(f,n)
 #else
 #include <security/pam_modutil.h>
 #include <security/pam_ext.h>
-#define fgetpwnam(h,f,n) _fgetpwnam_r(h,f,n)
-#define fgetspnam(h,f,n) _fgetspnam_r(h,f,n)
 #endif
 
-/* We have to include lckpwdf.-c as lckpwdf works hardcoded in /etc only */
-#include "./lckpwdf.-c"
+#if defined(USE_LCKPWDF) && !defined(HAVE_LCKPWDF)
+# include "./lckpwdf.-c"
+#endif
 
 static void
 strip_hpux_aging(char *hash)
@@ -122,7 +117,7 @@ verify_pwd_hash(const char *p, char *hash, unsigned int nullok)
 		p = NULL;		/* no longer needed here */
 
 		/* the moment of truth -- do we agree with the password? */
-		D(("comparing state of pp[%s] and hash[%s]", pp, hash));
+		D(("comparing state of pp[%s] and salt[%s]", pp, salt));
 
 		if (pp && strcmp(pp, hash) == 0) {
 			retval = PAM_SUCCESS;
@@ -154,191 +149,18 @@ is_pwd_shadowed(const struct passwd *pwd)
 	return 0;
 }
 
-#define PWD_INITIAL_LENGTH	0x100
-#define PWD_ABSURD_PWD_LENGTH	0x8000
-
-#if defined(HAVE_FGETPWENT_R) || defined(HAVE_FGETSPENT_R)
-#ifndef HELPER_COMPILE
-static void
-_cleanup(pam_handle_t *pamh UNUSED, void *data, int error_status UNUSED)
-{
-	free(data);
-}
-
-static int intlen(int number)
-{
-	int len = 2;
-	while (number != 0) {
-		number /= 10;
-		len++;
-	}
-	return len;
-}
-
-static int
-_pam_add_getnam_data(pam_handle_t *pamh, void *data,
-		     const char *modname, const char *user)
-{
-	char *data_name;
-	const void *ignore;
-	int i;
-	size_t len;
-	int status = PAM_NO_MODULE_DATA;
-
-	len = strlen(modname) + strlen(user) + 1 + intlen(INT_MAX) + 1;
-	data_name = malloc(len);
-	if ((pamh != NULL) && (data_name == NULL)) {
- 		D(("was unable to register the data item [%s]",
-		   pam_strerror(pamh, status)));
-		return PAM_BUF_ERR;
-	}
-	if (pamh != NULL) {
-		for (i = 0; i < INT_MAX; i++) {
-			snprintf(data_name, len, "%s%s_%d", modname, user, i);
-			status = PAM_NO_MODULE_DATA;
-			if (pam_get_data(pamh, data_name, &ignore) != PAM_SUCCESS) {
-				status = pam_set_data(pamh, data_name, data, _cleanup);
-			}
-			if (status == PAM_SUCCESS) {
-				break;
-			}
-		}
-	} else {
-		status = PAM_SUCCESS;
-	}
-
-	free(data_name);
-	return status;
-}
-#endif
-#endif
-
-/* Macros to avoid copy & paste. Alas, not much nicer. */
-
-#define FNAME(pre, kind, post)	pre##kind##post
-
-#define DEFINE_FGETXNAM(TYPE, KIND, PWNAME, FILENAME)				\
-static TYPE *									\
-FNAME(_fget, KIND, nam)(const char *filename, const char *user)			\
-{										\
-	FILE *pwfile;								\
-	TYPE *result = NULL;							\
-										\
-	if (strcmp(FILENAME, filename) == 0)					\
-		return FNAME(get, KIND, nam)(user);				\
-	else {									\
-		pwfile = fopen(filename, "r");					\
-		if (pwfile != NULL) {						\
-			while ((result = FNAME(fget, KIND, ent)(pwfile)) != NULL) \
-				if (strcmp(result->PWNAME, user) == 0)		\
-					break;					\
-			fclose(pwfile);						\
-		}								\
-	}									\
-	return result;								\
-}
-
-#define DEFINE_FGETXNAM_R(TYPE, KIND, PWNAME, FILENAME, MODNAME)		\
-static TYPE *									\
-FNAME(_fget, KIND, nam_r)(pam_handle_t *pamh, const char *filename, const char *user) \
-{										\
-	FILE *pwfile;								\
-	TYPE *result = NULL;							\
-	void *buffer = NULL;							\
-	size_t length = PWD_INITIAL_LENGTH;					\
-										\
-	if (strcmp(FILENAME, filename) == 0)					\
-		return FNAME(pam_modutil_get, KIND, nam)(pamh, user);		\
-										\
-	D(("file: %s", filename));						\
-	pwfile = fopen(filename, "r");						\
-	if (pwfile == NULL) {							\
-		D(("Cannot open file %s", filename));				\
-		return NULL;							\
-	}									\
-										\
-	do {									\
-		int status;							\
-		void *new_buffer;						\
-										\
-		new_buffer = realloc(buffer, sizeof(TYPE) + length);		\
-		if (new_buffer == NULL) {					\
-			D(("out of memory"));					\
-			goto finish;						\
-		}								\
-		buffer = new_buffer;						\
-										\
-		/* make the re-entrant call to get the structure */		\
-		errno = 0;							\
-		status = FNAME(fget, KIND, ent_r)(pwfile, buffer,		\
-				     (char *) buffer + sizeof(TYPE), length,	\
-				     &result);					\
-		if (!status) {							\
-			D(("comparing username %s to %s", user, result->PWNAME));\
-			if (strcmp(result->PWNAME, user) != 0)			\
-				continue;					\
-										\
-			status = _pam_add_getnam_data(pamh, buffer, MODNAME, user);\
-			if (status == PAM_SUCCESS) {				\
-				fclose(pwfile);					\
-				return result;					\
-			}							\
-			goto finish;						\
-		}								\
-		D(("fgetXent_r failed with status code %d (bufsize %d)",	\
-		   errno, sizeof(TYPE) + length));				\
-		   								\
-		length <<= 2;							\
-										\
-	} while (length < PWD_ABSURD_PWD_LENGTH);				\
-										\
-	D(("structure took %u bytes or so of memory",				\
-	   length+sizeof(TYPE)));						\
-finish:										\
-	if (buffer != NULL)							\
-    		free(buffer);							\
-	fclose(pwfile);								\
-	return NULL;								\
-}
-
-
-#if defined(HELPER_COMPILE) || !defined(HAVE_FGETPWENT_R)
-DEFINE_FGETXNAM(struct passwd, pw, pw_name, "/etc/passwd")
-#endif
-
-#ifndef HELPER_COMPILE
-#ifdef HAVE_FGETPWENT_R
-DEFINE_FGETXNAM_R(struct passwd, pw, pw_name, "/etc/passwd", "_pam_unix_fgetpwnam_")
+#ifdef HELPER_COMPILE
+int
+get_account_info(const char *name,
+	struct passwd **pwd, struct spwd **spwdent)
 #else
-static struct passwd *
-_fgetpwnam_r(pam_handle_t *pamh UNUSED, const char *filename, const char *user)
-{
-	return _fgetpwnam(filename, user);
-}
-#endif 
+int
+get_account_info(pam_handle_t *pamh, const char *name,
+	struct passwd **pwd,  struct spwd **spwdent)
 #endif
-
-#if defined(HELPER_COMPILE) || !defined(HAVE_FGETSPENT_R)
-DEFINE_FGETXNAM(struct spwd, sp, sp_namp, "/etc/shadow")
-#endif /* ! HAVE_FGETSPENT_R */
-
-#ifndef HELPER_COMPILE
-#ifdef HAVE_FGETSPENT_R
-DEFINE_FGETXNAM_R(struct spwd, sp, sp_namp, "/etc/shadow", "_pam_unix_fgetspnam_")
-#else /* ! HAVE_FGETSPENT_R */
-static struct spwd *
-_fgetspnam_r(pam_handle_t *pamh UNUSED, const char *filename, const char *user)
-{
-	return _fgetspnam(filename, user);
-}
-#endif 
-#endif /* ! HELPER_COMPILE */
-
-PAMH_ARG_DECL(int get_account_info,
-	unsigned int ctrl, const char *name, struct passwd **pwd, struct spwd **spwdent)
 {
 	/* UNIX passwords area */
-	*pwd = fgetpwnam(pamh, PASSWD_FILE, name);	/* Get password file entry... */
+	*pwd = pam_modutil_getpwnam(pamh, name);	/* Get password file entry... */
 	*spwdent = NULL;
 
 	if (*pwd != NULL) {
@@ -361,7 +183,7 @@ PAMH_ARG_DECL(int get_account_info,
 				}
 			}
 
-			*spwdent = fgetspnam(pamh, SHADOW_FILE, name);
+			*spwdent = pam_modutil_getspnam(pamh, name);
 			if (save_uid == (*pwd)->pw_uid)
 				setreuid(save_uid, save_euid);
 			else {
@@ -385,7 +207,7 @@ PAMH_ARG_DECL(int get_account_info,
 			if (geteuid() || SELINUX_ENABLED)
 				return PAM_UNIX_RUN_HELPER;
 #endif
-			*spwdent = fgetspnam(pamh, SHADOW_FILE, name);
+			*spwdent = pam_modutil_getspnam(pamh, name);
 			if (*spwdent == NULL || (*spwdent)->sp_pwdp == NULL)
 				return PAM_AUTHINFO_UNAVAIL;
 		}
@@ -395,13 +217,24 @@ PAMH_ARG_DECL(int get_account_info,
 	return PAM_SUCCESS;
 }
 
-PAMH_ARG_DECL(int get_pwd_hash,
-	unsigned int ctrl, const char *name, struct passwd **pwd, char **hash)
+#ifdef HELPER_COMPILE
+int
+get_pwd_hash(const char *name,
+	struct passwd **pwd, char **hash)
+#else
+int
+get_pwd_hash(pam_handle_t *pamh, const char *name,
+	struct passwd **pwd, char **hash)
+#endif
 {
 	int retval;
 	struct spwd *spwdent = NULL;
 
-	retval = get_account_info(PAMH_ARG(ctrl, name, pwd, &spwdent));
+#ifdef HELPER_COMPILE
+	retval = get_account_info(name, pwd, &spwdent);
+#else
+	retval = get_account_info(pamh, name, pwd, &spwdent);
+#endif
 	if (retval != PAM_SUCCESS) {
 		return retval;
 	}
@@ -416,13 +249,18 @@ PAMH_ARG_DECL(int get_pwd_hash,
 	return PAM_SUCCESS;
 }
 
-PAMH_ARG_DECL(int check_shadow_expiry,
-	struct spwd *spent, int *daysleft)
+#ifdef HELPER_COMPILE
+int
+check_shadow_expiry(struct spwd *spent, int *daysleft)
+#else
+int
+check_shadow_expiry(pam_handle_t *pamh, struct spwd *spent, int *daysleft)
+#endif
 {
 	long int curdays;
 	*daysleft = -1;
 	curdays = (long int)(time(NULL) / (60 * 60 * 24));
-	D(("today is %ld, last change %ld", curdays, spent->sp_lstchg));
+	D(("today is %d, last change %d", curdays, spent->sp_lstchg));
 	if ((curdays > spent->sp_expire) && (spent->sp_expire != -1)) {
 		D(("account expired"));
 		return PAM_ACCT_EXPIRED;
@@ -461,6 +299,10 @@ PAMH_ARG_DECL(int check_shadow_expiry,
 
 /* passwd/salt conversion macros */
 
+#define PW_TMPFILE              "/etc/npasswd"
+#define SH_TMPFILE              "/etc/nshadow"
+#define OPW_TMPFILE             "/etc/security/nopasswd"
+
 /*
  * i64c - convert an integer to a radix 64 character
  */
@@ -542,19 +384,17 @@ crypt_md5_wrapper(const char *pass_new)
         return cp;
 }
 
-PAMH_ARG_DECL(char * create_password_hash,
-	const char *password, unsigned int ctrl, int rounds)
+char *
+create_password_hash(const char *password, unsigned int ctrl, int rounds)
 {
 	const char *algoid;
 	char salt[64]; /* contains rounds number + max 16 bytes of salt + algo id */
 	char *sp;
 
 	if (on(UNIX_MD5_PASS, ctrl)) {
-		/* algoid = "$1" */
 		return crypt_md5_wrapper(password);
-	} else if (on(UNIX_BLOWFISH_PASS, ctrl)) {
-		algoid = "$2a$";
-	} else if (on(UNIX_SHA256_PASS, ctrl)) {
+	}
+	if (on(UNIX_SHA256_PASS, ctrl)) {
 		algoid = "$5$";
 	} else if (on(UNIX_SHA512_PASS, ctrl)) {
 		algoid = "$6$";
@@ -574,35 +414,17 @@ PAMH_ARG_DECL(char * create_password_hash,
 		return crypted;
 	}
 
-#ifdef HAVE_CRYPT_GENSALT_RN
-	if (on(UNIX_BLOWFISH_PASS, ctrl)) {
-		char entropy[17];
-		crypt_make_salt(entropy, sizeof(entropy) - 1);
-		sp = crypt_gensalt_rn(algoid, rounds,
-				      entropy, sizeof(entropy),
-				      salt, sizeof(salt));
-	} else {
-#endif
-		sp = stpcpy(salt, algoid);
-		if (on(UNIX_ALGO_ROUNDS, ctrl)) {
-			sp += snprintf(sp, sizeof(salt) - 3, "rounds=%u$", rounds);
-		}
-		crypt_make_salt(sp, 8);
-		/* For now be conservative so the resulting hashes
-		 * are not too long. 8 bytes of salt prevents dictionary
-		 * attacks well enough. */
-#ifdef HAVE_CRYPT_GENSALT_RN
+	sp = stpcpy(salt, algoid);
+	if (on(UNIX_ALGO_ROUNDS, ctrl)) {
+		sp += snprintf(sp, sizeof(salt) - 3, "rounds=%u$", rounds);
 	}
-#endif
+	crypt_make_salt(sp, 8);
+	/* For now be conservative so the resulting hashes
+	 * are not too long. 8 bytes of salt prevents dictionary
+	 * attacks well enough. */
 	sp = crypt(password, salt);
 	if (strncmp(algoid, sp, strlen(algoid)) != 0) {
-		/* libxcrypt/libc doesn't know the algorithm, use MD5 */
-		pam_syslog(pamh, LOG_ERR,
-			   "Algo %s not supported by the crypto backend, "
-			   "falling back to MD5\n",
-			   on(UNIX_BLOWFISH_PASS, ctrl) ? "blowfish" :
-			   on(UNIX_SHA256_PASS, ctrl) ? "sha256" :
-			   on(UNIX_SHA512_PASS, ctrl) ? "sha512" : algoid);
+	/* libc doesn't know the algorithm, use MD5 */
 		memset(sp, '\0', strlen(sp));
 		return crypt_md5_wrapper(password);
 	}
@@ -612,11 +434,11 @@ PAMH_ARG_DECL(char * create_password_hash,
 
 #ifdef WITH_SELINUX
 int
-unix_selinux_confined(unsigned int ctrl)
+unix_selinux_confined(void)
 {
     static int confined = -1;
     int fd;
-    char *tempfile = TEMPFILE;
+    char tempfile[]="/etc/.pwdXXXXXX";
 
     if (confined != -1)
     	return confined;
@@ -628,7 +450,7 @@ unix_selinux_confined(unsigned int ctrl)
     }
 
     /* let's try opening shadow read only */
-    if ((fd=open(SHADOW_FILE, O_RDONLY)) != -1) {
+    if ((fd=open("/etc/shadow", O_RDONLY)) != -1) {
         close(fd);
         confined = 0;
         return confined;
@@ -654,7 +476,7 @@ unix_selinux_confined(unsigned int ctrl)
 
 #else
 int
-unix_selinux_confined(unsigned int ctrl UNUSED)
+unix_selinux_confined(void)
 {
     return 0;
 }
@@ -662,13 +484,13 @@ unix_selinux_confined(unsigned int ctrl UNUSED)
 
 #ifdef USE_LCKPWDF
 int
-lock_pwdf(unsigned int ctrl)
+lock_pwdf(void)
 {
         int i;
         int retval;
 
 #ifndef HELPER_COMPILE
-        if (unix_selinux_confined(ctrl)) {
+        if (unix_selinux_confined()) {
                 return PAM_SUCCESS;
         }
 #endif
@@ -679,7 +501,7 @@ lock_pwdf(unsigned int ctrl)
            reasonable steps to make sure the token is updated; so retrying
            for 1/10 sec. isn't overdoing it. */
         i=0;
-        while((retval = __lckpwdf(ctrl)) != 0 && i < 100) {
+        while((retval = lckpwdf()) != 0 && i < 100) {
                 usleep(1000);
                 i++;
         }
@@ -690,32 +512,31 @@ lock_pwdf(unsigned int ctrl)
 }
 
 void
-unlock_pwdf(unsigned int ctrl)
+unlock_pwdf(void)
 {
 #ifndef HELPER_COMPILE
-        if (unix_selinux_confined(ctrl)) {
+        if (unix_selinux_confined()) {
                 return;
         }
 #endif
-        __ulckpwdf(ctrl);
+        ulckpwdf();
 }
 #else
 int
-lock_pwdf(unsigned int ctrl UNUSED)
+lock_pwdf(void)
 {
 	return PAM_SUCCESS;
 }
 
 void
-unlock_pwdf(unsigned int ctrl UNUSED)
+unlock_pwdf(void)
 {
 	return;
 }
 #endif
 
 int
-save_old_password(unsigned int ctrl,
-		  const char *forwho, const char *oldpass,
+save_old_password(const char *forwho, const char *oldpass,
 		  int howmany)
 {
     static char buf[16384];
@@ -745,7 +566,7 @@ save_old_password(unsigned int ctrl,
 #ifdef WITH_SELINUX
     if (SELINUX_ENABLED) {
       security_context_t passwd_context=NULL;
-      if (getfilecon(PASSWD_FILE,&passwd_context)<0) {
+      if (getfilecon("/etc/passwd",&passwd_context)<0) {
         return PAM_AUTHTOK_ERR;
       };
       if (getfscreatecon(&prev_context)<0) {
@@ -760,7 +581,7 @@ save_old_password(unsigned int ctrl,
       freecon(passwd_context);
     }
 #endif
-    pwfile = fopen(OLD_PASSWORDS_TMPFILE, "w");
+    pwfile = fopen(OPW_TMPFILE, "w");
     umask(oldmask);
     if (pwfile == NULL) {
       err = 1;
@@ -853,7 +674,7 @@ save_old_password(unsigned int ctrl,
 
 done:
     if (!err) {
-	if (rename(OLD_PASSWORDS_TMPFILE, OLD_PASSWORDS_FILE))
+	if (rename(OPW_TMPFILE, OLD_PASSWORDS_FILE))
 	    err = 1;
     }
 #ifdef WITH_SELINUX
@@ -869,13 +690,18 @@ done:
     if (!err) {
 	return PAM_SUCCESS;
     } else {
-	unlink(OLD_PASSWORDS_TMPFILE);
+	unlink(OPW_TMPFILE);
 	return PAM_AUTHTOK_ERR;
     }
 }
 
-PAMH_ARG_DECL(int unix_update_passwd,
-	unsigned int ctrl, const char *forwho, const char *towhat)
+#ifdef HELPER_COMPILE
+int
+unix_update_passwd(const char *forwho, const char *towhat)
+#else
+int
+unix_update_passwd(pam_handle_t *pamh, const char *forwho, const char *towhat)
+#endif
 {
     struct passwd *tmpent = NULL;
     struct stat st;
@@ -890,7 +716,7 @@ PAMH_ARG_DECL(int unix_update_passwd,
 #ifdef WITH_SELINUX
     if (SELINUX_ENABLED) {
       security_context_t passwd_context=NULL;
-      if (getfilecon(PASSWD_FILE,&passwd_context)<0) {
+      if (getfilecon("/etc/passwd",&passwd_context)<0) {
 	return PAM_AUTHTOK_ERR;
       };
       if (getfscreatecon(&prev_context)<0) {
@@ -905,14 +731,14 @@ PAMH_ARG_DECL(int unix_update_passwd,
       freecon(passwd_context);
     }
 #endif
-    pwfile = fopen(PASSWD_TMPFILE, "w");
+    pwfile = fopen(PW_TMPFILE, "w");
     umask(oldmask);
     if (pwfile == NULL) {
       err = 1;
       goto done;
     }
 
-    opwfile = fopen(PASSWD_FILE, "r");
+    opwfile = fopen("/etc/passwd", "r");
     if (opwfile == NULL) {
 	fclose(pwfile);
 	err = 1;
@@ -968,8 +794,12 @@ PAMH_ARG_DECL(int unix_update_passwd,
 
 done:
     if (!err) {
-	if (!rename(PASSWD_TMPFILE, PASSWD_FILE))
+	if (!rename(PW_TMPFILE, "/etc/passwd"))
+#ifdef HELPER_COMPILE
+	    helper_log_err(
+#else
 	    pam_syslog(pamh,
+#endif
 		LOG_NOTICE, "password changed for %s", forwho);
 	else
 	    err = 1;
@@ -987,13 +817,18 @@ done:
     if (!err) {
 	return PAM_SUCCESS;
     } else {
-	unlink(PASSWD_TMPFILE);
+	unlink(PW_TMPFILE);
 	return PAM_AUTHTOK_ERR;
     }
 }
 
-PAMH_ARG_DECL(int unix_update_shadow,
-	unsigned int ctrl, const char *forwho, char *towhat)
+#ifdef HELPER_COMPILE
+int
+unix_update_shadow(const char *forwho, char *towhat)
+#else
+int
+unix_update_shadow(pam_handle_t *pamh, const char *forwho, char *towhat)
+#endif
 {
     struct spwd *spwdent = NULL, *stmpent = NULL;
     struct stat st;
@@ -1013,7 +848,7 @@ PAMH_ARG_DECL(int unix_update_shadow,
 #ifdef WITH_SELINUX
     if (SELINUX_ENABLED) {
       security_context_t shadow_context=NULL;
-      if (getfilecon(SHADOW_FILE,&shadow_context)<0) {
+      if (getfilecon("/etc/shadow",&shadow_context)<0) {
 	return PAM_AUTHTOK_ERR;
       };
       if (getfscreatecon(&prev_context)<0) {
@@ -1028,14 +863,14 @@ PAMH_ARG_DECL(int unix_update_shadow,
       freecon(shadow_context);
     }
 #endif
-    pwfile = fopen(SHADOW_TMPFILE, "w");
+    pwfile = fopen(SH_TMPFILE, "w");
     umask(oldmask);
     if (pwfile == NULL) {
 	err = 1;
 	goto done;
     }
 
-    opwfile = fopen(SHADOW_FILE, "r");
+    opwfile = fopen("/etc/shadow", "r");
     if (opwfile == NULL) {
 	fclose(pwfile);
 	err = 1;
@@ -1089,8 +924,12 @@ PAMH_ARG_DECL(int unix_update_shadow,
 
  done:
     if (!err) {
-	if (!rename(SHADOW_TMPFILE, SHADOW_FILE))
+	if (!rename(SH_TMPFILE, "/etc/shadow"))
+#ifdef HELPER_COMPILE
+	    helper_log_err(
+#else
 	    pam_syslog(pamh,
+#endif
 		LOG_NOTICE, "password changed for %s", forwho);
 	else
 	    err = 1;
@@ -1110,7 +949,7 @@ PAMH_ARG_DECL(int unix_update_shadow,
     if (!err) {
 	return PAM_SUCCESS;
     } else {
-	unlink(SHADOW_TMPFILE);
+	unlink(SH_TMPFILE);
 	return PAM_AUTHTOK_ERR;
     }
 }
@@ -1118,14 +957,13 @@ PAMH_ARG_DECL(int unix_update_shadow,
 #ifdef HELPER_COMPILE
 
 int
-helper_verify_password(unsigned int ctrl,
-		       const char *name, const char *p, int nullok)
+helper_verify_password(const char *name, const char *p, int nullok)
 {
 	struct passwd *pwd = NULL;
 	char *salt = NULL;
 	int retval;
 
-	retval = get_pwd_hash(ctrl, name, &pwd, &salt);
+	retval = get_pwd_hash(name, &pwd, &salt);
 
 	if (pwd == NULL || salt == NULL) {
 		helper_log_err(LOG_WARNING, "check pass; user unknown");
